% This file perform the simulation for test for PARAMETER CONSTANCY in a GSTAR model
%
clear
clc
randn('state', 0);

%% STEP 0: Define the parameters
% Linear part
phi0_tilde = 0;
phi1_tilde = 0.8;
phi2_tilde = -0.7;
% NON-Linear part
theta0_tilde = 0;
theta1_tilde = -0.5;
theta2_tilde = 0.35;
%The slope parameters of time-smoothing variation (set zero for Size)
lambda1 = [phi0_tilde; phi1_tilde; phi2_tilde];
lambda2 = [theta0_tilde; theta1_tilde; theta2_tilde];
gamma11 = 5;
gamma21 = 2;
% Slope parameters of the G function in the main model
gamma1 = 200;
gamma2 = 100;
% Other parameters
T = 300;
nsim = 1000;
alpha01 = 0.01; alpha05 = 0.05; alpha10 = 0.1;
sigerr = 1;
d=100; % Discard the first 1,...d simulations

%Preallocate
Y = zeros(T - length([phi1_tilde phi2_tilde]),nsim);
LM1=zeros(1,nsim); LM2=zeros(1, nsim); LM3=zeros(1, nsim);
P_LM1=zeros(1, nsim); P_LM2=zeros(1, nsim); P_LM3=zeros(1, nsim);
Beta = zeros(1, nsim);
Power = zeros(1, nsim);

%% STEP2: INITIALIZATION
%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0_tilde, phi1_tilde, phi2_tilde, T , sigerr);
p = T-length(y);
%Generate G
[theta, eta, s, c, eta_t, h, mu, G] = sim_G_glstar1(phi0_tilde, phi1_tilde, phi2_tilde, theta0_tilde, theta1_tilde, theta2_tilde, gamma1, gamma2, y, T);

%Build up the elements of the Score and Hessian (for whom I need only the
%parameters gamma1 gamma2, T, eta and eta_t which in turns are function
%of the Linear AR model previously defined.
[Ggamma1] = Ggamma1(gamma1, gamma2, h, eta, eta_t, T);
[Ggamma2] = Ggamma2(gamma1, gamma2, h, eta, eta_t, T);
[Gc] = Gc(gamma1, gamma2, h, eta, eta_t, T);

% Set the definition for different types of asymmetry
t= T-p;
eta = (p+1:T)/T - c;                        %Monothonic function of parameters asymmetry
c10=c; c11=c10+1; c12=c10+2;
eta_c=(eta.^3 + c12*eta.^2 + c11*eta +c10); %Cubic function of parameters asymmetry
eps1 = eps(p+1:T);
%%  STEP3: SIMULATE a LINEAR AR(2) process BY USING PHI_tilde
for S=1:nsim
    
y = zeros(T-p,1);
%Definitions
%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0_tilde, phi1_tilde, phi2_tilde, T , sigerr);

%Generate G
[theta, eta, s, c, eta_t, h, mu, G]=sim_G_glstar1(phi0_tilde, phi1_tilde, phi2_tilde, theta0_tilde, theta1_tilde, theta2_tilde, gamma1, gamma2, y, T);

%Generate the NON-LINEAR part of the process in order to have w which is required for G and the
%Hessian
[y_nl, u_nl] = sim_nlAR(theta0_tilde, theta, G, T, sigerr);

%Define the GLSTAR1 as the sum of the Linear AR + Nonlinear AR previously
%defined
[y_gstar, u, sigma2_hat, sigma_hat] = nsim_GLSTAR1(T, y, y_nl, u_nl);

%%       STEP3: DEFINE THE TIME VARYING GSTAR
[tvh1,tvh2,tvh3,tvG1,tvG2,tvG3,phit_1,phit_2,phit_3,thetat_1,thetat_2,thetat_3]...
    = tv_par(eta,eta_c,phi0_tilde, phi1_tilde, phi2_tilde, theta0_tilde, theta1_tilde, theta2_tilde, gamma11,gamma21,lambda1,lambda2,T,p);

[y_tv1gstar,y_tv2gstar,y_tv3gstar,u_tv1gstar,u_tv2gstar,u_tv3gstar,Z] = tv_GSTAR(y,G,phit_1,phit_2,phit_3,thetat_1,thetat_2,thetat_3,eps1,T,p);

%%      STEP5: ESTIMATE THE TIME-VARYING Parameters by concentrated OLS 
%                   (Leybourne-Newbold-Vougas (1999)
%       
%Define the matrix of regressors
O=ones(1,p+1);
ZG=Z'.*kron(O,G);
ZGgamma1=Z'.*kron(O,Ggamma1);
ZGgamma2=Z'.*kron(O,Ggamma2);
ZGc=Z'.*kron(O,Gc);
%Regress and store the etimates and residuals
tvparams_ols = ols(u_tv1gstar, [Z' ZG]);
tvparams_est = [tvparams_ols(1:end).beta];
RES = [tvparams_ols(1:end).resid];
SE = [tvparams_ols(1:end).bstd];
SSR=sum(RES.^2);
sigma2 = (1/T)*SSR;

phi0_hat = tvparams_est(1);
phi1_hat = tvparams_est(2);
phi2_hat = tvparams_est(3);
theta0_hat = tvparams_est(4);
theta1_hat = tvparams_est(5);
theta2_hat = tvparams_est(6);

phi_hat = [phi0_hat; phi1_hat; phi2_hat];
theta_hat = [theta0_hat; theta1_hat; theta2_hat];

y_hattvgstar = phi_hat'*Z*(1-G) + theta_hat'*Z*G + eps1 ;
u_hattvgstar = y_hattvgstar - phi_hat'*Z*(1-G) - theta_hat'*Z*G;

%%       STEP5: TEST
%Define U and ZT=[t,t,t]  
k = length(lambda1);  
l = length(lambda2);
U = u_hattvgstar;
t = (1:T-p)';
ZT = kron(O,t);
z=[Z' ZG ZGgamma1, ZGgamma2, ZGc];
%Matrix V for the null of constancy with GLSTAR time variation
V2 = [ ZT.*Z'  ZT.*ZG];
%Matrix V for the null of constancy with GESTAR time variation
V1 = [ ZT.*Z' (ZT.^2).*Z'  ZT.*ZG (ZT.^2).*ZG];
%Matrix V for the null of constancy with UNKNOWN time variation
V3 = [];
%Matrix V for the ALTERNATIVE HP
V = [ ZT.*Z' (ZT.^2).*Z' (ZT.^3).*Z'  ZT.*ZG (ZT.^2).*ZG (ZT.^3).*ZG];
% Define the Covariate matrix
z01 = [V1 z]; % Matrix of covariates UNDER THE NULL of MONOTHONIC ASYMMETRY. 
z02 = [V2 z]; % Matrix of covariates UNDER THE NULL of NON-MONOTHONIC ASYMMETRY. 
z03 = [V3 z]; % Matrix of covariates UNDER THE NULL of NO ADDITITIVE ASYMMETRY.                      
z1 = [V z]; % Matrix of covariates UNDER THE ALTERNATIVE. 

%Regress and store the etimates and residuals under H0 - 
%NOTE: WE USE THE HETERO-ROBUST VERSION of
%the test while regressing U=[hat_u_tvgstar(t)] on z.
params01_ols = ols(U, z01);                    params02_ols = ols(U, z02);                   params03_ols = ols(U, z03);
params01_est = [params01_ols(1:end).beta];     params02_est = [params02_ols(1:end).beta];    params03_est = [params03_ols(1:end).beta];
RES01 = [params01_ols(1:end).resid];           RES02 = [params02_ols(1:end).resid];          RES03 = [params03_ols(1:end).resid];
SE01 = [params01_ols(1:end).bstd];             SE02 = [params02_ols(1:end).bstd];            SE03 = [params03_ols(1:end).bstd];
SSR01 = sum(RES01.^2);                         SSR02 = sum(RES02.^2);                        SSR03 = sum(RES03.^2);
sigma201 = (1/T)*SSR01;                        sigma202 = (1/T)*SSR02;                       sigma203 = (1/T)*SSR03;

%Regress and store the etimates and residuals under H1
tvparams13_ols = ols(U, z1);
tvparams13_est = [tvparams13_ols(1:end).beta];
RES13 = [tvparams13_ols(1:end).resid];
SE113 = [tvparams13_ols(1:end).bstd];
SSR1 = sum(RES13.^2);
sigma21 = (1/T)*SSR1;               

%   Chi2 statistic and p-values
%LM1 = (SSR01-SSR1)/sigma21; pval_LM1 = 1-cdf('chi2', LM1 , (k+l));
%LM2 = (SSR02-SSR1)/sigma21; pval_LM2 = 1-cdf('chi2', LM2 , (k+l));
%LM3 = (SSR03-SSR1)/sigma21; pval_LM3 = 1-cdf('chi2', LM3 , (k+l));

%   F-statistic and p-value 
%n=length(z);
LM1 = ((SSR01-SSR1)/((k+l))/(SSR1/(T-(k+l)-p))); pval_LM1 = 1-cdf('f', LM1, (k+l), T-(k+l)-p);
LM2 = ((SSR02-SSR1)/((k+l))/(SSR1/(T-(k+l)-p))); pval_LM2 = 1-cdf('f', LM2, (k+l), T-(k+l)-p);
LM3 = ((SSR03-SSR1)/((k+l))/(SSR1/(T-(k+l)-p))); pval_LM3 = 1-cdf('f', LM3, (k+l), T-(k+l)-p);

%Store the Draws of the Statistic and p-value
Y(:,S) = y_hattvgstar';
P_LM1(1,S) = pval_LM1;
P_LM2(1,S) = pval_LM2;
P_LM3(1,S) = pval_LM3;
end
%Discard the first d draws
Y(:,1:d)=[];
P_LM1(:,1:d)=[]; P_LM2(:,1:d)=[];P_LM3(:,1:d)=[];
%Define the Empirical P-value
EPval_LM1= mean(P_LM1); EPval_LM2= mean(P_LM2); EPval_LM3= mean(P_LM3);
%Define the Type II error (Beta) 
NS=1:size(Y,2);
Beta_P01_LM1=cumsum(P_LM1>alpha01)./NS; Beta_P01_LM2=cumsum(P_LM2>alpha01)./NS; Beta_P01_LM3=cumsum(P_LM3>alpha01)./NS; 
Beta_P05_LM1=cumsum(P_LM1>alpha05)./NS; Beta_P05_LM2=cumsum(P_LM2>alpha05)./NS; Beta_P05_LM3=cumsum(P_LM3>alpha05)./NS;
Beta_P10_LM1=cumsum(P_LM1>alpha10)./NS; Beta_P10_LM2=cumsum(P_LM2>alpha10)./NS; Beta_P10_LM3=cumsum(P_LM3>alpha10)./NS;
%  Power
Power_P01_LM1 = 1-Beta_P01_LM1;  Power_P01_LM2 = 1-Beta_P01_LM2; Power_P01_LM3 = 1-Beta_P01_LM3;
Power_P05_LM1 = 1-Beta_P05_LM1;  Power_P05_LM2 = 1-Beta_P05_LM2; Power_P05_LM3 = 1-Beta_P05_LM3;
Power_P10_LM1 = 1-Beta_P10_LM1;  Power_P10_LM2 = 1-Beta_P10_LM2; Power_P10_LM3 = 1-Beta_P10_LM3;
%  Empirical Beta
EBeta_P01_LM1 = mean(Beta_P01_LM1); EBeta_P01_LM2 = mean(Beta_P01_LM2); EBeta_P01_LM3 = mean(Beta_P01_LM3);
EBeta_P05_LM1 = mean(Beta_P05_LM1); EBeta_P05_LM2 = mean(Beta_P05_LM2); EBeta_P05_LM3 = mean(Beta_P05_LM3);
EBeta_P10_LM1 = mean(Beta_P10_LM1); EBeta_P10_LM2 = mean(Beta_P10_LM2); EBeta_P10_LM3 = mean(Beta_P10_LM3);
%  Empirical Power
EP_01_LM1 = mean(Power_P01_LM1); EP_01_LM2 = mean(Power_P01_LM2); EP_01_LM3 = mean(Power_P01_LM3);
EP_05_LM1 = mean(Power_P05_LM1); EP_05_LM2 = mean(Power_P05_LM2); EP_05_LM3 = mean(Power_P05_LM3);
EP_10_LM1 = mean(Power_P10_LM1); EP_10_LM2 = mean(Power_P10_LM2); EP_10_LM3 = mean(Power_P10_LM3);


%%%% PLOT THE RESULTS %%%%%%%%
disp('***************************************************************************************************')
disp('                                      Monte Carlo Results                                          ')
disp('***************************************************************************************************')
disp('---------------------------------------------------------------------------------------------------')
disp('                                          alpha = 0.01                                             ')
disp('---------------------------------------------------------------------------------------------------')
disp('               LM_01           ||            LM_02           ||            LM_03             ')
disp('-------------------------------||----------------------------||------------------------------')
disp(' Empirical  |  Emp.   |   Emp. ||   Emp   |  Emp.  |   Emp.  ||   Emp   |  Emp.  |   Emp.   ')
disp('  p-value   |  Beta   |  Power || p-value |  Beta  |  Power  || p-value |  Beta  |  Power   ')
disp('    ------  | ------  | ------ || ------  | ------ |  ------ || ------- | ------ |  ------  ')
disp( [EPval_LM1 EBeta_P01_LM1 EP_01_LM1 EPval_LM2 EBeta_P01_LM2 EP_01_LM2 EPval_LM3 EBeta_P01_LM3 EP_01_LM3])
disp('---------------------------------------------------------------------------------------------------')
disp('                                          alpha = 0.05                                             ')
disp('---------------------------------------------------------------------------------------------------')
disp('               LM_01           ||            LM_02           ||             LM_03             ')
disp('------------------------------ ||----------------------------||-------------------------------')
disp(' Empirical  |  Emp.   |   Emp. ||   Emp   |  Emp.  |   Emp.  ||  Emp    |  Emp.  |   Emp.  ')
disp('  p-value   |  Beta   |  Power || p-value |  Beta  |  Power  || p-value |  Beta  |  Power  ')
disp('    ------  | ------  | ------ || ------  | ------ |  ------ || ------- | ------ |  ------ ')
disp( [EPval_LM1 EBeta_P05_LM1 EP_05_LM1 EPval_LM2 EBeta_P05_LM2 EP_05_LM2 EPval_LM3 EBeta_P05_LM3 EP_05_LM3])
disp('---------------------------------------------------------------------------------------------------')
disp('                                        alpha = 0.10                                               ')
disp('---------------------------------------------------------------------------------------------------')
disp('               LM_01           ||           LM_02            ||            LM_03             ')
disp('------------------------------ ||----------------------------||------------------------------')
disp(' Empirical  |  Emp.   |   Emp. ||   Emp   |  Emp.  |  Emp.   ||   Emp   |  Emp.  |   Emp.   ')
disp('  p-value   |  Beta   |  Power || p-value |  Beta  | Power   || p-value |  Beta  |  Power   ')
disp('    ------  | ------  | ------ || ------  | ------ | ------  || ------- | ------ |  ------   ')
disp( [EPval_LM1 EBeta_P10_LM1 EP_10_LM1 EPval_LM2 EBeta_P10_LM2 EP_10_LM2 EPval_LM3 EBeta_P10_LM3 EP_10_LM3])
disp('----------------------------------------------------------------------------------------------------')

%Plot the power and the Beta
figure(1)
plot(NS, Beta_P01_LM1, NS, Power_P01_LM1,'-o');
legend('\beta  for  LM1', 'Power  for  LM1');