% This file perform the simulation for test for NO ERROR AUTOCORRELATION in a GSTAR model
%
clear
clc
randn('state', 7654321);
%% STEP 0: Define the parameters
%Linear Part
phi0 = 0.0;
phi1 = .8;
phi2 = -0.7;
%Nonlinear Linear Part
theta0 = 0.01;
theta1 = -0.9;
theta2 = 0.795;
%Slopes for different tails
gamma1 = 200;
gamma2 = 100;
% Serial correlation parameters
rho = 0;             % simulated correlation (set to zero for Size)
q = 1;                 % order of autocorrelation 
%Other parameter for simulation
T = 300;
nsim = 1000;
alpha01 = 0.01; alpha05 = 0.05; alpha10 = 0.1;
sigerr = 1.0; 
d= 100; %Discard the first d simulations

%Preallocate
Y = zeros(T-2,nsim);
P = zeros(1, nsim);
Beta = zeros(1, nsim); 
Power = zeros(1, nsim); 

%%                           STEP1: INITIALIZATION
%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0, phi1, phi2, T , sigerr);

%Generate G
[theta, eta, s, c, eta_t, h, mu, G] = sim_G_glstar1(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);

%Build up the elements of the Score and Hessian (for whom I need only the
%parameters gamma1 gamma2, T, eta and eta_t which in turns are function
%of the Linear AR model previously defined.
[Ggamma1] = Ggamma1(gamma1, gamma2, h , eta, eta_t, T);
[Ggamma2] = Ggamma2(gamma1, gamma2, h,  eta, eta_t, T);
[Gc] = Gc(gamma1, gamma2, h, eta, eta_t, T);

%%                      STEP2: SIMULATE
for S=1:nsim
    

%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0, phi1, phi2, T , sigerr);
u_l=u_l(3:end,1);
 
%Generate G
[theta, eta, s, c, eta_t, h, mu, G]=sim_G_glstar1(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);

%Generate the NON-LINEAR part of the process in order to have w which is required for G and the
%Hessian
[y_nl, u_nl] = sim_nlAR(theta0, theta, G, T, sigerr);

%Define the GLSTAR1 as the sum of the Linear AR + Nonlinear AR previously
%defined
[y_gstar, u, sigma2_hat, sigma_hat] = nsim_GLSTAR1(T, y, y_nl, u_nl);
Y(:,S)=y_gstar;

%%       STEP 3: DEFINE THE GSTAR under Serial correlation
%Simulate the GSTAR process with SERIALLY CORRELATED ERRORS following a 
%GLSTAR model
[y_gstar, u_scgstar, y_scgstar, p, bar_z, Z, v] = sim_scGSTAR(y, phi0, phi, theta0, theta, T, G, eps, q, rho);


%%  STEP4: ESTIMATE THE GSTAR MODEL Parameters by concentrated OLS 
%                   (Leybourne-Newbold-Vougas (1999)
      
%Define the matrix of regressors
O=ones(1,p+1);
ZG=Z'.*kron(O,G);

%Regress and store the etimates and residuals of the model generated UNDER 
%THE NULL HYPOTHESIS OF NO ERROR AUTOCORRELATION
params_ols = ols(y_scgstar', [Z' ZG Ggamma1, Ggamma2, Gc]);
params_est = [params_ols(1:end).beta];
RES = [params_ols(1:end).resid];
SE = [params_ols(1:end).bstd];
SSR=sum(RES.^2);
sigma2 = (1/T)*SSR;

phi0_hat = params_est(1);
phi1_hat = params_est(2);
phi2_hat = params_est(3);
theta0_hat = params_est(4);
theta1_hat = params_est(5);
theta2_hat = params_est(6);

phi_hat = [phi0_hat; phi1_hat; phi2_hat];
theta_hat = [theta0_hat; theta1_hat; theta2_hat];

eps1=eps(p+1:end)';

y_hatscgstar = phi_hat'*Z + theta_hat'*Z.*G' + eps1 ; %Estimated y_gstar(t) under SC
u_hatscgstar = y_hatscgstar - phi_hat'*Z - theta_hat'*Z.*G'; %Estimated u_gstar(t) under SC

%Define the vector v_hat
v_hat = zeros(length(u_hatscgstar),q);   %preallocate the vector v_hat
for j=1:q;
    v_hat(:,j) = lag(u_hatscgstar,j) ;   %define v=[u_gstar(t-1),...,u_gstar(t-q)]
end

%%                     STEP 5: TEST

%Define the vectors 
U = u_hatscgstar';
z = [v, Z', ZG, Ggamma1, Ggamma2, Gc];

%  F-statistic
%Regress and store the etimates and residuals under H1 (The estimates under
%H0 has been done previously)

params1_ols = ols(U, z);
params1_est = [params1_ols(1:end).beta];
RES1 = [params1_ols(1:end).resid];
SE1 = [params1_ols(1:end).bstd];
SSR1=sum(RES1.^2);
sigma21 = (1/T)*SSR1;

%F-statistic
n = size([Z', ZG, Ggamma1, Ggamma2, Gc],2);
%LM = (SSR-SSR1)/(sigma21);
LM = ((SSR-SSR1)/(q))/(SSR1/(T-q-n));

%Compute the p-value according to the so found statistics
%pval = 1-cdf('chi2', LM , q);
pval = 1-cdf('f', LM, q, T-q-n);

%Store the Draws of the Statistic and p-value
Y(:,S) = y_hatscgstar';
P(1,S) = pval; 
end
%Discard the first d draws
Y(:,1:d)=[];
P(:,1:d)=[];
%Define the Empirical P-value
EPval= mean(P);
%Define the Type II error (Beta)
NS=1:size(Y,2);
Beta_P_01=cumsum(P>alpha01)./NS; 
Beta_P_05=cumsum(P>alpha05)./NS; 
Beta_P_10=cumsum(P>alpha10)./NS; 
%  Power
Power_P_01= 1-Beta_P_01; 
Power_P_05= 1-Beta_P_05; 
Power_P_10= 1-Beta_P_10; 
%  Empirical Beta
EBeta_P_01=mean(Beta_P_01); 
EBeta_P_05=mean(Beta_P_05); 
EBeta_P_10=mean(Beta_P_10); 
%  Empirical Power
EP_P_01=mean(Power_P_01); 
EP_P_05=mean(Power_P_05); 
EP_P_10=mean(Power_P_10); 

%%%% PLOT THE RESULTS %%%%%%%%
disp('***************************************************************************************************')
disp('                                      Monte Carlo Results                                          ')
disp('***************************************************************************************************')
disp('---------------------------------------------------------------------------------------------------')
disp('              alpha = 0.01       ')
disp('---------------------------------')
disp('                 q=1             ')
disp('---------------------------------')
disp(' Empirical  |  Emp.   |   Emp.   ')
disp('  p-value   |  Beta   |  Power   ')
disp('    ------  | ------  | ------   ')
disp( [EPval EBeta_P_01 EP_P_01 ])
disp('---------------------------------')
disp('           alpha = 0.05          ')
disp('---------------------------------')
disp('                          ')
disp('             ')
disp('-------------------------------  ')
disp(' Empirical  |  Emp.   |   Emp.   ')
disp('  p-value   |  Beta   |  Power   ')
disp('    ------  | ------  | ------   ')
disp( [EPval EBeta_P_05 EP_P_05])
disp('---------------------------------')
disp('          alpha = 0.10           ')
disp('---------------------------------')
disp('                          ')
disp('                  ')
disp('-------------------------------  ')
disp(' Empirical  |  Emp.   |   Emp.   ')
disp('  p-value   |  Beta   |  Power   ')
disp('    ------  | ------  | ------   ')
disp( [EPval EBeta_P_10 EP_P_10 ])
disp('---------------------------------')
%Plot the power and the Beta
%figure(1)
%plot(NS, Beta_P_01, NS, Power_P_01,'-o');
%legend('Beta', 'Power');