% 
% This file define the function "h(gamma)" in Stukel (1988) for a fixed "eta" 
%
function [h, MU,G] = h_gamma(eta, gamma1, gamma2)
%This function performs the Generalized Logistic function by Stukel (1988)

%Define the behavior of the function for various gammas
h11 = (gamma1^(-1)).*(exp(gamma1*abs(eta)) - 1) ; 
h12 = eta ;
h13 = (-gamma1^(-1))*log(1 - gamma1*abs(eta)) ; 

h21 = (-gamma2^(-1)).*(exp(gamma2*abs(eta)) - 1) ;
h22 = eta ;
h23 = (gamma2^(-1))*log(1 - gamma2*abs(eta)) ; 

% Assign the function h for each gamma
if gamma1 > 0
    h1 = h11;
elseif gamma1 == 0
    h1 = h12;
else h1 = h13;
end

if gamma2 > 0
    h2 = h21;
elseif gamma2 == 0
    h2 = h22 ;
else h2 = h23;
end

%Summarize the function for plotting
h = (eta<=0).*(h2) + (eta>=0).*(h1);
MU = exp(h)./(1 + exp(h));
G = (1 + exp(-h)).^-1;
end