function [y_gstar, uadd, sigma2_hat, sigma_hat] = nsim_GLSTAR1add(T, y, y_nl, y_nladd, u_l, u_nl, u_nladd)
%This function simulate a time series following the GLSTAR1 model WITH AN
%ADDITIVE ASYMMETRY
%
%The process is: y_t = phi_0 + phi_1 y_t-1 + phi_2*y_t-2 + (theta01 + theta11 + theta21)*G1  + 
%                      +(theta02 + theta12 + theta22)*G2+ eps.
%                G is an AR2 process
%
%
%Simulate the GSTAR(1) model like an additive model (a linear AR + a 
%Nonlinear AR) and compute estimated residuals, variance of
%estimated residuals and sd:
y_gstar=zeros(size(y,1),1);
uadd=zeros(size(y,1),1);
for i = 1 : T-2
    y_gstar(i) = y(i) + y_nl(i) + y_nladd(i) ;
    uadd(i) = u_l(i) + u_nl(i) + u_nladd(i);   %compute the estimated residuals as the sum 
                               %of the residuals of the two parts
end

sigma2_hat = (1/T)*sum((uadd.^2));  %compute the variance of the residuals 
sigma_hat = sqrt(sigma2_hat);    %compute the SD of the residuals




end