% This script reproduces Figure 2 in Supplement of Zanetti Chini (2018)
%
clear
clc
randn('state', 1234);

phi0= 0.05;
phi1= .4;
phi2= .25;
theta0= 0.2;
theta1= .4;
theta2= .25;
gamma1 = .5;
gamma2 = -2.0;
c=0; 
c1=3;
c2=5;
T = 100;
nsim = 1;

%Simulate 
y = random('normal',0,2,T,1); %(the variance is setted 4 in order to have a Transition function with good property)
%Derive other variables
z=[ones(T,1),lag(y,1),lag(y,2)]';                 % Covariates matrix
s=lag(y,1);                                       % Transition variable s_t = y_t-d
c=repmat(c,length(s),1);                          %The scale parameter c is vectorized;                   
phi=[phi0, phi1, phi2]'; theta=[theta0, theta1, theta2]';
eta=s-c; eta1=s-c1;                              %Two different simulations of eta are used for didactical scope
%Compute two different G function with EZC parametrization
[h, MU, G] = G_ezc(eta, gamma1, gamma2);
[h1, MU1, G1] = G_ezc(eta1, gamma1, gamma2);
%Compute two different G function with STUKEL parametrization
[eta, h_stuk, G_stuk] = G_stukel(eta, gamma1, gamma2);
[eta1, h_stuk1, G_stuk1] = G_stukel(eta1, gamma1, gamma2);
%Compute two different G function for SYMMETRIC STAR 
Gsymm = (1 + exp(-(gamma1)*(s-c))).^(-1);   % The traditional symmetric version of the STAR model
Gsymm1 = (1 + exp(-(gamma1)*(s-c1))).^(-1); % The traditional symmetric version of the STAR model
%Compute some other Functions:
eta_lin = (phi'*z).*(1-G');             % The "Linear Part" of the model
eta_nl_stuk = (theta'*z).*G_stuk';      % The "NON-Linear Part" of the STUKEL-GSTAR
eta_nl_ezc = (theta'*z).*G';            % The "NON-Linear Part" of the EZC-GSTAR
eta_gstar_stuk = eta_lin + eta_nl_stuk; % The outcome of the STUKEL-GSTAR 
eta_gstar_ezc = eta_lin + eta_nl_ezc;   % The outcome of the EZC-GSTAR

%% OUTPUT

figure(1)
subplot(2,2,1)
P1=plot(1:T,eta_lin','-ko',1:T,eta_gstar_stuk,'-',1:T,eta_gstar_ezc,'r-');
xlabel('T')                          
ylabel('\eta_t')                       
title('SIMULATED PROCESS')
legend('\phi^T\theta\times (1-G)','S-GSTAR', 'EZC-GSTAR', 1)
set(P1(1),'Linewidth',.5); set(P1(2),'LineWidth',2.0); set(P1(3),'Linewidth',1.0);

subplot(2,2,2)
P2=plot(1:T,G_stuk,'-ro',1:T,G,'b-');
xlabel('T')                          
ylabel('G(.,.,.)')                       
title('TRANSITION FUNCTION')
legend('Stukel G-function' , 'EZC G-function', 1)
set(P2(1),'Linewidth',1.0); set(P2(2),'LineWidth',2.0);

subplot(2,2,3)
P=plot(sort(eta),sort(G_stuk),'-k', sort(eta),sort(G_stuk1),'-r', sort(eta),sort(G),'s',sort(eta),sort(G1),'o', sort(eta),sort(Gsymm),'-mv',sort(eta),sort(Gsymm1),'-ch');
xlabel('\eta_t')                          
ylabel('G(.,.,.)') 
title('TRANSITION FUNCTION vs ORDERED \eta_t')
legend('Gstukel','Gstukel1' , 'Gezc', 'Gezc1', 'Gsymm', 'Gsymm1', 2)
set(P(1),'Linewidth',2); set(P(2),'Linewidth',2);set(P(3),'MarkerSize',10); set(P(4),'MarkerSize',10);

subplot(2,2,4)
P4=plot(sort(eta),sort(h_stuk),sort(eta),sort(h));
xlabel('\eta_t')                          
ylabel('h(\eta_t)')                       
title('h(\eta_t) vs ORDERED \eta_t')
legend('Stukel h-function', 'EZC h-function', 2)
set(P4(1),'Linewidth',2.0); set(P4(2),'LineWidth',2.0);