function [S1, pval_S1] = stest_aio(y_gstar, T, s)
%This funtion computes the "All-in-One" test for symmetry for applications
%
%Step 0: Define the covariate matrix under the NULL HYPOTHESIS OF LINEARITY
zh0 = -[ones(size(y_gstar,1),1),lag(y_gstar,1),lag(y_gstar,2)];

%Step 0-bis: define the matrix of augmented auxiliary model in case of asymmetry
z_nl = [zh0(:,2:end).*repmat(s,1,2),zh0(:,2:end).*repmat(s.^2,1,2),zh0(:,2:end).*repmat(s.^3,1,2)];

%Step 0-ter: Define the covariate matrix under the ALTERNATIVE HYPOTHESIS OF ASYMMETRY
zh1 = [zh0, z_nl];

%Step 1: regress y on the covariate matrix under the null of linearity and compute the SSR0
results0=ols(y_gstar, zh0);
RES0=[results0(1:end).resid];
SSR0=sum(RES0.^2);
sigma20 = (1/T)*SSR0;
    
%Step 2: Regress y on the COVARIANCE MATRIX UNDER ALTERNATIVE 
%(that is estimate the true Asymmetric model) and compute the SSR1
results1 = ols(RES0, zh1);
RES1 = [results1(1:end).resid];
SSR1 = sum(RES1.^2);
sigma21 = (1/T)*SSR1;

%Step 3: built the test statistics LM2 using F-test and compute p-value
n = size(zh1,2);
p = size(zh0,2);
S1 = (SSR0-SSR1)/sigma21;
%S1 = ((SSR0-SSR1)/(3*p+n))/(SSR1/(T-3*p-n));
CDF = cdf('chi2', S1, 3*p);
%CDF = cdf('f', S1, 3*p+n, T-3*p-n);
pval_S1 = 1-CDF;
end